@@ -38,10 +38,6 @@ int main (int argc, char **argv)
   LIBMTP_mtpdevice_t *device;
   LIBMTP_file_t *files;
   uint32_t xmlfileid = 0;
-  uint64_t totalbytes;
-  uint64_t freebytes;
-  char *storage_description;
-  char *volume_label;
   char *friendlyname;
   char *syncpartner;
   char *sectime;
@@ -97,32 +93,6 @@ int main (int argc, char **argv)
     free(syncpartner);
   }
 
-  // Some storage info
-  ret = LIBMTP_Get_Storageinfo(device, &totalbytes, &freebytes, &storage_description, &volume_label);
-  if (ret == 0) {
-#ifdef __WIN32__
-    printf("   Total bytes on device: %I64u (%I64u MB)\n",
-	   totalbytes, totalbytes/(1024*1024));
-    printf("   Free bytes on device: %I64u (%I64u MB)\n",
-	   freebytes, freebytes/(1024*1024));
-#else
-    printf("   Total bytes on device: %llu (%llu MB)\n",
-	   totalbytes, totalbytes/(1024*1024));
-    printf("   Free bytes on device: %llu (%llu MB)\n",
-	   freebytes, freebytes/(1024*1024));
-#endif
-    if (storage_description != NULL) {
-      printf("   Storage description: \"%s\"\n", storage_description);
-      free(storage_description);
-    }
-    if (volume_label != NULL) {
-      printf("   Volume label: \"%s\"\n", volume_label);
-      free(volume_label);
-    }
-  } else {
-    printf("   Error getting disk info...\n");
-  }
-
   // Some battery info
   ret = LIBMTP_Get_Batterylevel(device, &maxbattlevel, &currbattlevel);
   if (ret == 0) {
@@ -158,7 +128,7 @@ int main (int argc, char **argv)
   }
 
   // Try to get Media player device info XML file...
-  files = LIBMTP_Get_Filelisting(device);
+  files = LIBMTP_Get_Filelisting_With_Callback(device, NULL, NULL);
   if (files != NULL) {
     LIBMTP_file_t *file, *tmp;
     file = files;
