@@ -10,7 +10,7 @@
 #ifndef LIBMTP_H_INCLUSION_GUARD
 #define LIBMTP_H_INCLUSION_GUARD
 
-#define LIBMTP_VERSION 0.0.21
+#define LIBMTP_VERSION 0.1.3
 
 /* This handles MSVC pecularities */
 #ifdef _MSC_VER
@@ -74,6 +74,19 @@ typedef enum {
   LIBMTP_FILETYPE_WINEXEC,
   LIBMTP_FILETYPE_TEXT,
   LIBMTP_FILETYPE_HTML,
+  LIBMTP_FILETYPE_FIRMWARE,
+  LIBMTP_FILETYPE_AAC,
+  LIBMTP_FILETYPE_MEDIACARD,
+  LIBMTP_FILETYPE_FLAC,
+  LIBMTP_FILETYPE_MP2,
+  LIBMTP_FILETYPE_M4A,
+  LIBMTP_FILETYPE_DOC,
+  LIBMTP_FILETYPE_XML,
+  LIBMTP_FILETYPE_XLS,
+  LIBMTP_FILETYPE_PPT,
+  LIBMTP_FILETYPE_MHT,
+  LIBMTP_FILETYPE_JP2,
+  LIBMTP_FILETYPE_JPX,
   LIBMTP_FILETYPE_UNKNOWN
 } LIBMTP_filetype_t;
 typedef struct LIBMTP_device_entry_struct LIBMTP_device_entry_t; /**< @see LIBMTP_device_entry_struct */
@@ -81,12 +94,15 @@ typedef struct LIBMTP_mtpdevice_struct LIBMTP_mtpdevice_t; /**< @see LIBMTP_mtpd
 typedef struct LIBMTP_file_struct LIBMTP_file_t; /**< @see LIBMTP_file_struct */
 typedef struct LIBMTP_track_struct LIBMTP_track_t; /**< @see LIBMTP_track_struct */
 typedef struct LIBMTP_playlist_struct LIBMTP_playlist_t; /**< @see LIBMTP_playlist_struct */
+typedef struct LIBMTP_album_struct LIBMTP_album_t; /**< @see LIBMTP_album_struct */
 typedef struct LIBMTP_folder_struct LIBMTP_folder_t; /**< @see LIBMTP_folder_t */
 typedef struct LIBMTP_object_struct LIBMTP_object_t; /**< @see LIBMTP_object_t */
+typedef struct LIBMTP_filesampledata_struct LIBMTP_filesampledata_t; /**< @see LIBMTP_filesample_t */
+typedef struct LIBMTP_devicestorage_struct LIBMTP_devicestorage_t; /**< @see LIBMTP_devicestorage_t */
 
-/** 
+/**
  * The callback type definition. Notice that a progress percentage ratio
- * is easy to calculate by dividing <code>sent</code> by 
+ * is easy to calculate by dividing <code>sent</code> by
  * <code>total</code>.
  * @param sent the number of bytes sent so far
  * @param total the total number of bytes to send
@@ -97,8 +113,8 @@ typedef struct LIBMTP_object_struct LIBMTP_object_t; /**< @see LIBMTP_object_t *
 typedef int (* LIBMTP_progressfunc_t) (uint64_t const sent, uint64_t const total,
                 		void const * const data);
 
-/** 
- * @} 
+/**
+ * @}
  * @defgroup structar libmtp data structures
  * @{
  */
@@ -119,18 +135,22 @@ struct LIBMTP_device_entry_struct {
 struct LIBMTP_mtpdevice_struct {
   /** Interface number of this device */
   uint8_t interface_number;
-  /** 
-   * Parameters for this device, must be cast into 
+  /**
+   * Parameters for this device, must be cast into
    * \c (PTPParams*) before internal use.
    */
   void *params;
-  /** 
+  /**
    * USB device for this device, must be cast into
    * \c (PTP_USB*) before internal use.
    */
   void *usbinfo;
-  /** The storage ID for this device */
-  uint32_t storage_id;
+  /** 
+   * The storage for this device, do not use strings in here without 
+   * copying them first, and beware that this list may be rebuilt at
+   * any time.
+   */
+  LIBMTP_devicestorage_t *storage;
   /** The maximum battery level for this device */
   uint8_t maximum_battery_level;
   /** Default music folder */
@@ -145,6 +165,10 @@ struct LIBMTP_mtpdevice_struct {
   uint32_t default_organizer_folder;
   /** Default ZENcast folder (only Creative devices...) */
   uint32_t default_zencast_folder;
+  /** Default Album folder */
+  uint32_t default_album_folder;
+  /** Default Text folder */
+  uint32_t default_text_folder;
   /** Per device iconv() converters, only used internally */
   void *cd;
 };
@@ -166,6 +190,7 @@ struct LIBMTP_file_struct {
  */
 struct LIBMTP_track_struct {
   uint32_t item_id; /**< Unique item ID */
+  uint32_t parent_id; /**< ID of parent folder */
   char *title; /**< Track title */
   char *artist; /**< Name of recording artist */
   char *genre; /**< Genre name for track */
@@ -174,7 +199,7 @@ struct LIBMTP_track_struct {
   char *filename; /**< Original filename of this track */
   uint16_t tracknumber; /**< Track number (in sequence on recording) */
   uint32_t duration; /**< Duration in milliseconds */
-  uint32_t samplerate; /**< Sample rate of original file, min 0x1f80 max 0xbb80 */	
+  uint32_t samplerate; /**< Sample rate of original file, min 0x1f80 max 0xbb80 */
   uint16_t nochannels; /**< Number of channels in this recording 0 = unknown, 1 or 2 */
   uint32_t wavecodec; /**< FourCC wave codec name */
   uint32_t bitrate; /**< (Average) bitrate for this file min=1 max=0x16e360 */
@@ -197,6 +222,17 @@ struct LIBMTP_playlist_struct {
   LIBMTP_playlist_t *next; /**< Next playlist or NULL if last playlist */
 };
 
+/**
+ * MTP Album structure
+ */
+struct LIBMTP_album_struct {
+  uint32_t album_id; /**< Unique playlist ID */
+  char *name; /**< Name of album */
+  uint32_t *tracks; /**< The tracks in this album */
+  uint32_t no_tracks; /**< The number of tracks in this album */
+  LIBMTP_album_t *next; /**< Next album or NULL if last album */
+};
+
 /**
  * MTP Folder structure
  */
@@ -209,18 +245,34 @@ struct LIBMTP_folder_struct {
 };
 
 /**
- * LIBMTP Object Structure
+ * LIBMTP Object RepresentativeSampleData Structure
  */
-struct LIBMTP_object_struct { 
-  uint32_t id; 
-  uint32_t parent; 
-  uint32_t type; 
-  uint32_t size;
-  char *name; 
-  void *data; 
-  LIBMTP_object_t *sibling; 
-  LIBMTP_object_t *child; 
-}; 
+struct LIBMTP_filesampledata_struct {
+  uint32_t width; /**< Width of sample if it is an image */
+  uint32_t height; /**< Height of sample if it is an image */
+  uint32_t duration; /**< Duration in milliseconds if it is audio */
+  LIBMTP_filetype_t filetype; /**< Filetype used for the sample */
+  uint64_t size; /**< Size of sample data in bytes */
+  char *data; /**< Sample data */
+};
+
+/**
+ * LIBMTP Device Storage structure
+ */
+struct LIBMTP_devicestorage_struct {
+  uint32_t id; /**< Unique ID for this storage */
+  uint16_t StorageType; /**< Storage type */
+  uint16_t FilesystemType; /**< Filesystem type */
+  uint16_t AccessCapability; /**< Access capability */
+  uint64_t MaxCapacity; /**< Maximum capability */
+  uint64_t FreeSpaceInBytes; /**< Free space in bytes */
+  uint64_t FreeSpaceInObjects; /**< Free space in objects */
+  char *StorageDescription; /**< A brief description of this storage */
+  char *VolumeIdentifier; /**< A volume identifier */
+  LIBMTP_devicestorage_t *next; /**< Next storage, follow this link until NULL */
+  LIBMTP_devicestorage_t *prev; /**< Previous storage */
+};
+  
 
 /** @} */
 
@@ -251,17 +303,21 @@ char *LIBMTP_Get_Friendlyname(LIBMTP_mtpdevice_t*);
 int LIBMTP_Set_Friendlyname(LIBMTP_mtpdevice_t*, char const * const);
 char *LIBMTP_Get_Syncpartner(LIBMTP_mtpdevice_t*);
 int LIBMTP_Set_Syncpartner(LIBMTP_mtpdevice_t*, char const * const);
-int LIBMTP_Get_Storageinfo(LIBMTP_mtpdevice_t *, uint64_t * const, 
-			uint64_t * const, char ** const storage_description, 
-			char ** const volume_label);
-int LIBMTP_Format_Storage(LIBMTP_mtpdevice_t *);
-int LIBMTP_Get_Batterylevel(LIBMTP_mtpdevice_t *, 
-			    uint8_t * const, 
+int LIBMTP_Get_Batterylevel(LIBMTP_mtpdevice_t *,
+			    uint8_t * const,
 			    uint8_t * const);
 int LIBMTP_Get_Secure_Time(LIBMTP_mtpdevice_t *, char ** const);
 int LIBMTP_Get_Device_Certificate(LIBMTP_mtpdevice_t *, char ** const);
 int LIBMTP_Get_Supported_Filetypes(LIBMTP_mtpdevice_t *, uint16_t ** const, uint16_t * const);
 
+#define LIBMTP_STORAGE_SORTBY_NOTSORTED 0
+#define LIBMTP_STORAGE_SORTBY_FREESPACE 1
+#define LIBMTP_STORAGE_SORTBY_MAXSPACE  2
+
+int LIBMTP_Get_Storage(LIBMTP_mtpdevice_t *, int const);
+int LIBMTP_Format_Storage(LIBMTP_mtpdevice_t *, LIBMTP_devicestorage_t *);
+
+
 /**
  * @}
  * @defgroup files The file management API.
@@ -271,17 +327,27 @@ LIBMTP_file_t *LIBMTP_new_file_t(void);
 void LIBMTP_destroy_file_t(LIBMTP_file_t*);
 char const * LIBMTP_Get_Filetype_Description(LIBMTP_filetype_t);
 LIBMTP_file_t *LIBMTP_Get_Filelisting(LIBMTP_mtpdevice_t *);
+LIBMTP_file_t *LIBMTP_Get_Filelisting_With_Callback(LIBMTP_mtpdevice_t *,
+      LIBMTP_progressfunc_t const, void const * const);
 LIBMTP_file_t *LIBMTP_Get_Filemetadata(LIBMTP_mtpdevice_t *, uint32_t const);
-int LIBMTP_Get_File_To_File(LIBMTP_mtpdevice_t*, uint32_t, char const * const, 
+int LIBMTP_Get_File_To_File(LIBMTP_mtpdevice_t*, uint32_t, char const * const,
 			LIBMTP_progressfunc_t const, void const * const);
-int LIBMTP_Get_File_To_File_Descriptor(LIBMTP_mtpdevice_t*, uint32_t const, int const, 
+int LIBMTP_Get_File_To_File_Descriptor(LIBMTP_mtpdevice_t*, uint32_t const, int const,
 			LIBMTP_progressfunc_t const, void const * const);
-int LIBMTP_Send_File_From_File(LIBMTP_mtpdevice_t *, char const * const, 
+int LIBMTP_Send_File_From_File(LIBMTP_mtpdevice_t *, char const * const,
 	                 LIBMTP_file_t * const, LIBMTP_progressfunc_t const,
 			 void const * const, uint32_t const);
-int LIBMTP_Send_File_From_File_Descriptor(LIBMTP_mtpdevice_t *, int const, 
+int LIBMTP_Send_File_From_File_Descriptor(LIBMTP_mtpdevice_t *, int const,
 	                LIBMTP_file_t * const, LIBMTP_progressfunc_t const,
 			void const * const, uint32_t const);
+LIBMTP_filesampledata_t *LIBMTP_new_filesampledata_t(void);
+void LIBMTP_destroy_filesampledata_t(LIBMTP_filesampledata_t *);
+int LIBMTP_Get_Representative_Sample_Format(LIBMTP_mtpdevice_t *,
+                        LIBMTP_filetype_t const,
+                        LIBMTP_filesampledata_t **);
+int LIBMTP_Send_Representative_Sample(LIBMTP_mtpdevice_t *, uint32_t const,
+                          LIBMTP_filesampledata_t *);
+
 /**
  * @}
  * @defgroup tracks The track management API.
@@ -290,20 +356,22 @@ int LIBMTP_Send_File_From_File_Descriptor(LIBMTP_mtpdevice_t *, int const,
 LIBMTP_track_t *LIBMTP_new_track_t(void);
 void LIBMTP_destroy_track_t(LIBMTP_track_t*);
 LIBMTP_track_t *LIBMTP_Get_Tracklisting(LIBMTP_mtpdevice_t*);
+LIBMTP_track_t *LIBMTP_Get_Tracklisting_With_Callback(LIBMTP_mtpdevice_t*,
+      LIBMTP_progressfunc_t const, void const * const);
 LIBMTP_track_t *LIBMTP_Get_Trackmetadata(LIBMTP_mtpdevice_t*, uint32_t const);
-int LIBMTP_Get_Track_To_File(LIBMTP_mtpdevice_t*, uint32_t, char const * const, 
+int LIBMTP_Get_Track_To_File(LIBMTP_mtpdevice_t*, uint32_t, char const * const,
 			LIBMTP_progressfunc_t const, void const * const);
-int LIBMTP_Get_Track_To_File_Descriptor(LIBMTP_mtpdevice_t*, uint32_t const, int const, 
+int LIBMTP_Get_Track_To_File_Descriptor(LIBMTP_mtpdevice_t*, uint32_t const, int const,
 			LIBMTP_progressfunc_t const, void const * const);
-int LIBMTP_Send_Track_From_File(LIBMTP_mtpdevice_t *, 
+int LIBMTP_Send_Track_From_File(LIBMTP_mtpdevice_t *,
 			 char const * const, LIBMTP_track_t * const,
                          LIBMTP_progressfunc_t const,
 			 void const * const, uint32_t const);
-int LIBMTP_Send_Track_From_File_Descriptor(LIBMTP_mtpdevice_t *, 
+int LIBMTP_Send_Track_From_File_Descriptor(LIBMTP_mtpdevice_t *,
 			 int const, LIBMTP_track_t * const,
                          LIBMTP_progressfunc_t const,
 			 void const * const, uint32_t const);
-int LIBMTP_Update_Track_Metadata(LIBMTP_mtpdevice_t *, 
+int LIBMTP_Update_Track_Metadata(LIBMTP_mtpdevice_t *,
 			LIBMTP_track_t const * const);
 int LIBMTP_Track_Exists(LIBMTP_mtpdevice_t *, uint32_t);
 /** @} */
@@ -333,44 +401,27 @@ LIBMTP_playlist_t *LIBMTP_Get_Playlist(LIBMTP_mtpdevice_t *, uint32_t const);
 int LIBMTP_Create_New_Playlist(LIBMTP_mtpdevice_t *, LIBMTP_playlist_t * const, uint32_t const);
 int LIBMTP_Update_Playlist(LIBMTP_mtpdevice_t *, LIBMTP_playlist_t const * const);
 
+/**
+ * @}
+ * @defgroup albums The audio/video album management API.
+ * @{
+ */
+LIBMTP_album_t *LIBMTP_new_album_t(void);
+void LIBMTP_destroy_album_t(LIBMTP_album_t *);
+LIBMTP_album_t *LIBMTP_Get_Album_List(LIBMTP_mtpdevice_t *);
+LIBMTP_album_t *LIBMTP_Get_Album(LIBMTP_mtpdevice_t *, uint32_t const);
+int LIBMTP_Create_New_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t * const, uint32_t const);
+int LIBMTP_Update_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t const * const);
 
 /**
  * @}
  * @defgroup objects The object management API.
  * @{
  */
-char *LIBMTP_Get_String_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t);
-uint32_t LIBMTP_Get_U32_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint32_t);
-uint16_t LIBMTP_Get_U16_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint16_t);
-int LIBMTP_Set_Object_String(LIBMTP_mtpdevice_t *,const uint32_t,const uint16_t, 
-                             char const * const);
-int LIBMTP_Set_Object_U32(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint32_t);
-int LIBMTP_Set_Object_U16(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint16_t);
-int LIBMTP_Get_Object_References(LIBMTP_mtpdevice_t *, const uint32_t, uint32_t **, uint32_t *);
-int LIBMTP_Set_Object_References(LIBMTP_mtpdevice_t *, const uint32_t, uint32_t const * const, const uint32_t);
-LIBMTP_object_t *LIBMTP_Make_List(LIBMTP_mtpdevice_t *, uint32_t *, uint32_t, uint32_t *, uint32_t);
-LIBMTP_object_t *LIBMTP_Find_Object(LIBMTP_object_t *, const uint32_t);
-void LIBMTP_Dump_List(LIBMTP_object_t *);
-LIBMTP_object_t *LIBMTP_new_object_t(void);
-void LIBMTP_destroy_object_t(LIBMTP_object_t *, uint32_t);
 int LIBMTP_Delete_Object(LIBMTP_mtpdevice_t *, uint32_t);
-/** @} */
 
-/**
- * @}
- * @defgroup File mapping The file mapping API
- * @{
- */
-int LIBMTP_Register_Filetype(char const * const, LIBMTP_filetype_t const, 
-			     uint16_t const, void const * const, 
-			     void const * const, void const * const);
-int LIBMTP_Set_Filetype_Description(LIBMTP_filetype_t, char const * const);
-int LIBMTP_Set_Constructor(LIBMTP_filetype_t, void const * const);
-int LIBMTP_Set_Destructor(LIBMTP_filetype_t, void const * const);
-int LIBMTP_Set_Datafunc(LIBMTP_filetype_t, void const * const);
 /** @} */
 
-
 /* End of C++ exports */
 #ifdef __cplusplus
 }
